import { NgModule } from '@angular/core';
import { of, Observable } from 'rxjs';

// noinspection JSUnusedGlobalSymbols
class ExcTranslation {
    constructor() {
        this.translateService = ExcCoreModule.Services().TranslateService;
    }
    currentLang() {
        if (this.translateService) {
            return this.translateService.language;
        }
        return null;
    }
    onLangChange() {
        if (this.translateService) {
            return this.translateService.onLangChange;
        }
        return of();
    }
    // noinspection SpellCheckingInspection
    getRessource(name, param) {
        return this.getResource(name, param);
    }
    getResource(name, param, reroutelist) {
        if (this.translateService) {
            let rtVal = (this.translateService.get(this.constructor.name + '.' + name, param)).value;
            if (typeof rtVal === typeof '') {
                if (rtVal.indexOf('$$Translation$$') === 0) {
                    rtVal = this.getTranslation(rtVal.replace('$$Translation$$', ''), param);
                }
                else if (rtVal.indexOf('$$Global$$') === 0) {
                    rtVal = this.getGlobal(rtVal.replace('$$Global$$', ''), param);
                }
            }
            return rtVal;
        }
        return 'NoService' + this.constructor.name + '.' + name;
    }
    getGlobal(name, param, reroutelist) {
        if (this.translateService) {
            let rtVal = (this.translateService.get('Globals.' + name, param)).value;
            if (rtVal === '$$Global$$' + name) {
                return rtVal.replace('$$Global$$', 'Globals.');
            }
            if (typeof rtVal === typeof '') {
                if (rtVal.indexOf('$$Translation$$') === 0) {
                    rtVal = this.getTranslation(rtVal.replace('$$Translation$$', ''), param);
                }
                else if (rtVal.indexOf('$$Global$$') === 0) {
                    rtVal = this.getGlobal(rtVal.replace('$$Global$$', ''), param);
                }
            }
            return rtVal;
        }
        return 'NoService' + 'Globals.' + name;
    }
    getTranslation(name, param) {
        if (this.translateService) {
            let rtVal = (this.translateService.get(name, param)).value;
            if (rtVal === '$$Translation$$' + name) {
                return rtVal.replace('$$Translation$$', '');
            }
            if (typeof rtVal === typeof '') {
                if (rtVal.indexOf('$$Translation$$') === 0) {
                    rtVal = this.getTranslation(rtVal.replace('$$Translation$$', ''), param);
                }
                else if (rtVal.indexOf('$$Global$$') === 0) {
                    rtVal = this.getGlobal(rtVal.replace('$$Global$$', ''), param);
                }
            }
            return rtVal;
        }
        return 'NoService' + name;
    }
}
// noinspection JSUnusedGlobalSymbols
class ExcCoreModule {
    static on(name, callback) {
        if (ExcCoreModule._events[name] == null) {
            ExcCoreModule._events[name] = [];
            ExcCoreModule._unload[name] = [];
        }
        ExcCoreModule._events[name].push(callback);
        const unloadFunc = () => {
            const index = ExcCoreModule._events[name].indexOf(callback);
            ExcCoreModule._events[name].splice(index, 1);
            ExcCoreModule._unload[name].splice(index, 1);
        };
        ExcCoreModule._unload[name].push(unloadFunc);
        return unloadFunc;
    }
    static call(name, ...args) {
        const data = args;
        if (ExcCoreModule._events[name] == null) {
            return;
        }
        for (let i = 0; i < ExcCoreModule._events[name].length; i++) {
            const _c = ExcCoreModule._events[name].length;
            ExcCoreModule._events[name][i].apply(ExcCoreModule._unload[name][i], data);
            if (_c > ExcCoreModule._events[name].length) {
                i--;
            }
        }
    }
    static lastToastRemove(newID) {
        if (ExcCoreModule.lastToastId) {
            const el = document.getElementById(`exc-tst-${ExcCoreModule.lastToastId}`);
            if (el) {
                el.remove();
            }
        }
        ExcCoreModule.lastToastId = newID;
    }
    static showToast(message, classname, icon) {
        const id = ExcCoreModule.Guid.NewGuid();
        ExcCoreModule.lastToastRemove(id);
        const html = `<div class="${classname} animate" id="exc-tst-${id}"><i class="${icon}"></i>${message}</div>`;
        document.body.insertAdjacentHTML('afterbegin', html);
        const el = document.getElementById(`exc-tst-${id}`);
        if (el === null) {
            return;
        }
        setTimeout(() => {
            el.remove();
        }, 1500);
    }
    static showToastCommit(message, classname, commitText, commitCallbackFn, icon) {
        const id = ExcCoreModule.Guid.NewGuid();
        ExcCoreModule.lastToastRemove(id);
        const html = `<div class="${classname}" id="exc-tst-${id}">
                  <i class="${icon}"></i>${message}
                    <button id="exc-tst-${id}-btn">${commitText}</button>
                  </div>`;
        document.body.insertAdjacentHTML('afterbegin', html);
        const el = document.getElementById(`exc-tst-${id}`);
        const elBtn = document.getElementById(`exc-tst-${id}-btn`);
        if (el === null || elBtn === null) {
            return;
        }
        const evLstnr = () => {
            if (commitCallbackFn) {
                commitCallbackFn();
            }
            elBtn.removeEventListener('click', evLstnr);
            el.remove();
        };
        elBtn.addEventListener('click', evLstnr);
    }
    static showInfo(message, title) {
        ExcCoreModule.showToast(message, 'md-toast', undefined);
    }
    static showSuccess(message, title) {
        ExcCoreModule.showToast(message, 'md-toast', undefined);
    }
    static showWarning(message, title) {
        ExcCoreModule.showToast(message, 'md-toast', undefined);
    }
    static showError(message, title) {
        ExcCoreModule.showToastCommit(message, 'md-toast', 'OK', undefined, undefined);
    }
    /**
     * Open a Modal Window
     * @param content Text or component to be shown in modal window
     * @param header Text to be shown in modal header
     * @param size Size of the modal window ('xs','s','m','l','xl')
     * @param data Data to be passed to modal window in case of component use
     * @param closeViaOverlay true|false (default:false) close modal per outbound click
     * @param footerButtons array of button declaration to be shown in footer button-format({text, class, onClick()})
     */
    static showModal(content, header, size, data, closeViaOverlay = false, footerButtons) {
        const excTranslation = new ExcTranslation();
        return new Observable(obs => {
            const config = {
                settings: {
                    size: size || 's',
                    header: (typeof (header) === 'string'),
                    resources: {
                        headerText: header
                    },
                    closableViaOverlay: closeViaOverlay,
                    onClose: (result) => {
                        obs.next(result || false);
                    }
                },
                // tslint:disable-next-line:object-literal-shorthand
                data: data
            };
            if (typeof (content) === 'function') {
                config.settings.footer = false;
                config.settings.templateButtons = false;
                if (Array.isArray(footerButtons)) {
                    config.settings.footer = true;
                    config.settings.footerButtons = footerButtons;
                }
                if (footerButtons === true) {
                    config.settings.footer = true;
                    config.settings.templateButtons = true;
                    config.settings.footerButtons = undefined;
                }
            }
            else {
                config.settings.footer = true;
                if (footerButtons) {
                    config.settings.footerButtons = footerButtons;
                }
                else {
                    config.settings.footerButtons = [
                        {
                            text: excTranslation.getGlobal('No'),
                            class: 'md-btn',
                            onClick: (overlay) => {
                                overlay.close(false);
                            }
                        }, {
                            text: excTranslation.getGlobal('Yes'),
                            class: 'md-btn-solid',
                            onClick: (overlay) => {
                                overlay.close(true);
                            }
                        }
                    ];
                }
            }
            ExcCoreModule.call('eOS.showModal', content, config);
        });
    }
    static Services() {
        return ExcCoreModule._services;
    }
    static Config() {
        return ExcCoreModule._config;
    }
    static RegisterService(key, value) {
        const _com = value;
        Object.defineProperty(ExcCoreModule._services, key, {
            get: () => {
                return _com;
            }, configurable: true
        });
    }
    static RegisterConfig(key, value) {
        const _com = value;
        Object.defineProperty(ExcCoreModule._config, key, {
            get: () => {
                return _com;
            }, configurable: true
        });
    }
    static s4() {
        return Math.floor((1 + Math.random()) * 0x10000)
            .toString(16)
            .substring(1)
            .toUpperCase();
    }
}
// Helper functions
ExcCoreModule.Array = {
    Get(arr, value) {
        return arr.find(e => e === value);
    },
    GetIndex(arr, value) {
        return arr.findIndex(e => e === value);
    },
    GetObject(arr, key, value) {
        return arr.find(e => e[key] === value);
    },
    GetObjectIndex(arr, key, value) {
        return arr.findIndex(e => e[key] === value);
    },
    GetAll(arr, key, value) {
        return arr.filter(e => e[key] === value);
    },
    Remove(arr, value) {
        arr.splice(arr.findIndex(e => e === value), 1);
        return arr;
    },
    RemoveAll(arr, value) {
        for (let index = arr.findIndex(e => e === value); index !== -1; index = arr.findIndex(e => e === value)) {
            arr.splice(index, 1);
        }
        return arr;
    },
    RemoveObject(arr, key, value) {
        arr.splice(arr.findIndex(e => e[key] === value), 1);
        return arr;
    },
    RemoveObjectAll(arr, key, value) {
        for (let index = arr.findIndex(e => e[key] === value); index !== -1; index = arr.findIndex(e => e[key] === value)) {
            arr.splice(index, 1);
        }
        return arr;
    },
    Add(arr, value) {
        arr.push(value);
        return arr;
    },
    Merge(arr, value) {
        if (!value || value.length === 0) {
            return arr;
        }
        for (const item of value) {
            arr.push(item);
        }
        /*for (let i = 0; i < value.length; i++) {
          arr.push(value[i]);
        }*/
        return arr;
    },
    Exists(arr, value) {
        return arr.findIndex(e => e === value) >= 0;
    },
    ExistsObject(arr, key, value) {
        return arr.findIndex(e => e[key] === value) >= 0;
    },
    Distinct(arr) {
        return arr.filter((obj, i, arr2) => arr2.findIndex(t => t === obj) === i);
    },
    DistinctObject(arr, key) {
        return arr.filter((obj, i, arr2) => arr2.findIndex(t => t[key] === obj[key]) === i);
    }
};
ExcCoreModule.Guid = {
    NewGuid() {
        return ExcCoreModule.s4() + ExcCoreModule.s4() + '-' + ExcCoreModule.s4() + '-' + ExcCoreModule.s4() + '-' +
            ExcCoreModule.s4() + '-' + ExcCoreModule.s4() + ExcCoreModule.s4() + ExcCoreModule.s4();
    }
};
ExcCoreModule.Object = {
    Replace(destination, source) {
        for (const property in source) {
            if (source.hasOwnProperty(property)) {
                if (destination.hasOwnProperty(property)) {
                    destination[property] = source[property];
                }
            }
        }
    },
    Merge(destination, ...source) {
        // tslint:disable-next-line:prefer-for-of
        for (let i = 0; i < source.length; i++) {
            const object = source[i];
            for (const property in object) {
                if (object.hasOwnProperty(property)) {
                    destination[property] = object[property];
                }
            }
        }
    },
    DeepMerge(destination, ...source) {
        // tslint:disable-next-line:prefer-for-of
        for (let i = 0; i < source.length; i++) {
            const sourceItem = source[i];
            for (const sourceItemProperty in sourceItem) {
                if (sourceItem.hasOwnProperty(sourceItemProperty)) {
                    if (destination[sourceItemProperty] === undefined) {
                        destination[sourceItemProperty] = sourceItem[sourceItemProperty];
                    }
                    else if (destination[sourceItemProperty] instanceof Array && sourceItem[sourceItemProperty] instanceof Array) {
                        for (const arrayItem of sourceItem[sourceItemProperty]) {
                            this.DeepMerge(destination[sourceItemProperty], sourceItem[sourceItemProperty]);
                        }
                    }
                    else if (typeof destination[sourceItemProperty] === typeof {} && typeof sourceItem[sourceItemProperty] === typeof {}) {
                        this.DeepMerge(destination[sourceItemProperty], sourceItem[sourceItemProperty]);
                    }
                    else {
                        destination[sourceItemProperty] = sourceItem[sourceItemProperty];
                    }
                }
            }
        }
    }
};
ExcCoreModule.String = {
    Format(text, ...args) {
        return text.replace(/{(\d+)}/g, (match, num) => {
            return typeof args[Number(num)] !== 'undefined'
                ? args[Number(num)]
                : match;
        });
    },
    Reverse(text) {
        let newString = '';
        for (let s = 0; s < text.length; s++) {
            newString = text.charAt(s) + newString;
        }
        return newString;
    }
};
ExcCoreModule._events = {};
ExcCoreModule._unload = {};
ExcCoreModule._services = {};
ExcCoreModule._config = {};
ExcCoreModule.lastToastId = null;
ExcCoreModule.decorators = [
    { type: NgModule, args: [{
                declarations: [],
                imports: [],
                exports: []
            },] }
];

/*
 * Public API Surface of exc-core-v2
 */

/**
 * Generated bundle index. Do not edit.
 */

export { ExcCoreModule, ExcTranslation };
//# sourceMappingURL=exc-core-v2.js.map
